@@ -28,7 +28,7 @@
 #' @param nreps Integer number of simulations to be carried out 
 #' @param ncpu Number of threads to use for parallel computation. Recommended.
 #' @param overrideTempConstraint If TRUE (default) will not enforce positive branch lengths in simualtion replicates. Will speed up execution. 
-#' @param overrideClock May be 'strict' or 'relaxed' in which case will force simulations to fit the corresponding model. If ommitted, will inherit the clock model from td
+#' @param overrideClock May be 'strict' or 'additive' or 'uncorrelated' in which case will force simulations to fit the corresponding model. If ommitted, will inherit the clock model from td
 #' @param overrideSearchRoot If TRUE, will re-use root position from input treedater tree. Otherwise may re-estimate root position in simulations
 #' @param overrideSeqLength Optional sequence length to use in simulations
 #' @param quiet If TRUE will minimize output printed to screen
@@ -59,7 +59,7 @@
 #' # modify edge length to represent evolutionary distance with rate 1e-3:
 #' tre$edge.length <- tre$edge.length * 1e-3
 #' # treedater: 
-#' td <- dater( tre, sts =sts )
+#' td <- dater( tre, sts=sts, s=1000, clock='strict', omega0=.0015 )
 #' # parametric bootstrap: 
 #' pb <- parboot( td, nreps=25 )
 #' # plot lineages through time
@@ -70,7 +70,7 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 {
 	k = 0 # resolve NOTE about 'visible binding for global variable'
 	if (quiet){
-	cat( 'Running in quiet mode. To print progress, set quiet=FALSE.\n')
+		cat( 'Running in quiet mode. To print progress, set quiet=FALSE.\n')
 	}
 	if (overrideSearchRoot){
 		cat('NOTE: Running with overrideSearchRoot will speed up execution but may underestimate variance.\n')
@@ -88,42 +88,48 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 			blen <- pmax( 1e-12, td$edge.length )
 			if( td$clock == 'strict' ) {
 				# simulate poisson 
-				tre$edge.length <- rpois(length(tre$edge.length), blen * td$meanRate * td$s ) / td$s
-			} else {
+				tre$edge.length <- rpois(length(tre$edge.length), blen * td$mean.rate * td$s ) / td$s
+			} else if (td$clock=='uncorrelated') {
 				#ps <- pmin(1 - 1e-5, td$theta*blen  / ( 1+ td$theta * blen ) )
 				ps <- pmin(1 - 1e-12, td$theta*blen  / ( 1+ td$theta * blen ) )
 				tre$edge.length <- rnbinom( length(td$edge.length)
 				 , td$r
 				 , 1 - ps
 				) / td$s
+			} else if ( td$clock=='additive'){
+				sizes = td$mu * blen / td$sp 
+				tre$edge.length <- rnbinom( length(td$edge.length)
+				 , sizes
+				 , 1 - td$sp / ( 1 + td$sp )
+				) / td$s
+			} else{
+				stop( '*clock* not supported' )
 			}
+			
 			if (!td$intree_rooted & !overrideSearchRoot) tre <- unroot( tre )
 			est <- NULL
 			if (td$EST_SAMP_TIMES) est <- td$estimateSampleTimes
 			tempConstraint <- td$temporalConstraints
 			if ( overrideTempConstraint) tempConstraint <- FALSE
-			clockstr <- td$clock
+			clock <- td$clock
 			if (!is.null( overrideClock)){
 				if (is.na(overrideClock)) stop('overrideClock NA. Stopping.')
-				if (!overrideClock %in% c('strict', 'relaxed') ){
-					stop('overrideClock must be one of strict or relaxed')
-				}
-				clockstr <- overrideClock
+				clock <- overrideClock
 			}
-			strictClock <- ifelse( clockstr=='strict' , TRUE, FALSE )
+			
 			seqlen <- ifelse( is.null(overrideSeqLength) , td$s, overrideSeqLength )
 			td2 <- tryCatch({dater(tre, td$sts, s= seqlen
 			 , omega0 = NA
 			 , minblen = td$minblen
 			 , quiet = TRUE
 			 , temporalConstraints = tempConstraint
-			 , strictClock = strictClock
+			 , clock = clock 
 			 , estimateSampleTimes = est
 			 , estimateSampleTimes_densities = td$estimateSampleTimes_densities
 			 , numStartConditions = td$numStartConditions 
 			 , meanRateLimits = td$meanRateLimits
-			)}, error =function(e) NA )
-			if (suppressWarnings( is.na( td2[1])) ) return (NA )
+			)}, error =function(e) NULL )
+			if (suppressWarnings( is.null( td2)) ) return (NULL )
 			if (!quiet){
 				cat('\n #############################\n')
 				cat( paste( '\n Replicate', k, 'complete \n' ))
@@ -139,26 +145,35 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 			if (!success) stop('*parallel_foreach* requires the `foreach` package. Stopping.')
 			`%dopar%` <- foreach::`%dopar%`
 			tds <- foreach::foreach( k = 1:nreps, .packages=c('treedater') ) %dopar% {
-				capture.output( { pbrk <- .parboot.replicate(k) })
+				if (quiet )
+					capture.output( { pbrk <- .parboot.replicate(k) })
+				else
+					 pbrk <- .parboot.replicate(k)
 				pbrk 
 			}
 		} else{
 			tds <- parallel::mclapply( 1:nreps, function(k) {
-				capture.output( { pbrk <- .parboot.replicate(k) })
+				if (quiet)
+					capture.output( { pbrk <- .parboot.replicate(k) })
+				else
+					pbrk <- .parboot.replicate(k)
 				pbrk
 			}
 			, mc.cores = ncpu ) 
 		}
 	} else{
 		tds <- lapply( 1:nreps, function(k) {
-			capture.output( { pbrk <- .parboot.replicate(k) })
+			if (quiet)
+				capture.output( { pbrk <- .parboot.replicate(k) })
+			else
+				pbrk <- .parboot.replicate(k)
 			pbrk
 		})
 	}
-	tds <- tds[!suppressWarnings ( sapply(tds, function(td) is.na(td[1]) ) )  ] 
+	tds <- tds[ !sapply(tds,  is.null)  ] 
 	if (length(tds)==0) stop('All bootstrap replicate failed with error.')
 	# output rate CIs, parameter CIs, trees
-	meanRates <- sapply( tds, function(td) td$meanRate )
+	meanRates <- unlist( sapply( tds, function(td) td$mean.rate ) )
 	cvs <- sapply( tds, function(td) td$coef_of_variation )
 	tmrcas <- sapply( tds, function(td) td$timeOfMRCA )
 	ttmrcas <- sapply( tds, function(td) td$timeTo )
@@ -175,7 +190,7 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 	rv <- list( 
 		trees = tds
 		, meanRates = meanRates
-		, meanRate_CI = c( exp( log(td$meanRate) - log_mr_sd*1.96), exp(log(td$meanRate) + log_mr_sd*1.96 ))
+		, meanRate_CI = c( exp( log(td$mean.rate) - log_mr_sd*1.96), exp(log(td$mean.rate) + log_mr_sd*1.96 ))
 		, coef_of_variation_CI = quantile( cvs, probs = c(alpha/2, 1 - alpha/2))
 		, timeOfMRCA_CI = timeOfMRCA_CI
 		, td = td
@@ -188,6 +203,18 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 	rv
 }
 
+# ensure that tip composition of btres is same as treedater tree 
+.boot.trees <- function (td, btres)
+{
+sapply( btres, function(btre ){
+ if ( length(setdiff(btre$tip.label, td$tip.label) )== 0)
+	if (length( setdiff( td$tip.label, btre$tip.label)) ==0)
+		return(TRUE)
+return(FALSE)
+}) -> x
+all(x)
+}
+
 
 #' Estimate of confidence intervals of molecular clock parameters with user-supplied set of bootstrap trees
 #'
@@ -206,7 +233,7 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 #' @param ncpu Number of threads to use for parallel computation. Recommended.
 #' @param searchRoot See *dater*
 #' @param overrideTempConstraint If TRUE (default) will not enforce positive branch lengths in simualtion replicates. Will speed up execution. 
-#' @param overrideClock May be 'strict' or 'relaxed' in which case will force simulations to fit the corresponding model. If ommitted, will inherit the clock model from td
+#' @param overrideClock May be 'strict' or 'additive' or 'relaxed' in which case will force simulations to fit the corresponding model. If ommitted, will inherit the clock model from td
 #' @param quiet If TRUE will minimize output printed to screen
 #' @param normalApproxTMRCA If TRUE will use estimate standard deviation from simulation replicates and report confidence interval based on normal distribution
 #' @param parallel_foreach If TRUE will use the foreach package for parallelization. May work better on HPC systems. 
@@ -241,7 +268,7 @@ parboot <- function( td , nreps = 100, ncpu = 1,  overrideTempConstraint=TRUE, o
 #' })
 #' tre$edge.length <- tre$edge.length * 1e-3
 #' # run treedater
-#' td <- dater( tre, sts  )
+#' td <- dater( tre, sts, s= 1000, clock='strict', omega0=.0015  )
 #' # bootstrap: 
 #' ( tdboot <- boot( td, bootTrees ) )
 #' # plot lineages through time :
@@ -274,22 +301,18 @@ boot <- function( td, tres,  ncpu = 1, searchRoot=1 , overrideTempConstraint=TRU
 		if (td$EST_SAMP_TIMES) est <- td$estimateSampleTimes
 		tempConstraint <- td$temporalConstraints
 		if ( overrideTempConstraint) tempConstraint <- FALSE
-		clockstr <- td$clock
+		clock <- td$clock
 		if (!is.null( overrideClock)){
 			if (is.na(overrideClock)) stop('overrideClock NA. Quitting.')
-			if (!overrideClock %in% c('strict', 'relaxed') ){
-				stop('overrideClock must be one of strict or relaxed')
-			}
-			clockstr <- overrideClock
+			clock <- overrideClock
 		}
-		strictClock <- ifelse( clockstr=='strict' , TRUE, FALSE )
 		td2 <- tryCatch({ dater(tre, td$sts, s= td$s
 		 , omega0 = NA
 		 , minblen = td$minblen
 		 , quiet = TRUE
 		 , searchRoot = searchRoot
 		 , temporalConstraints = tempConstraint
-		 , strictClock = strictClock
+		 , clock = clock 
 		 , estimateSampleTimes = est
 		 , estimateSampleTimes_densities = td$estimateSampleTimes_densities
 		 , numStartConditions = td$numStartConditions 
@@ -304,7 +327,7 @@ boot <- function( td, tres,  ncpu = 1, searchRoot=1 , overrideTempConstraint=TRU
 		}
 		td2
 	}
-
+	
 	if (ncpu > 1)
 	{
 		if (parallel_foreach){
@@ -312,26 +335,35 @@ boot <- function( td, tres,  ncpu = 1, searchRoot=1 , overrideTempConstraint=TRU
 			if (!success) stop('*parallel_foreach* requires the `foreach` package. Stopping.')
 			`%dopar%` <- foreach::`%dopar%`
 			tds <- foreach::foreach( k = 1:nreps, .packages=c('treedater') ) %dopar% {
+				if (quiet)
 				 capture.output( { brk <- .boot.replicate(k) })
+				else 
+					brk <- .boot.replicate(k)
 				 brk 
 			}
 		} else{
 			tds <- parallel::mclapply( 1:nreps, function(k) {
-				capture.output( { brk <- .boot.replicate(k) })
+				if (quiet )
+					capture.output( { brk <- .boot.replicate(k) })
+				else
+					brk <- .boot.replicate(k)
 				brk 
 			}
 			, mc.cores = ncpu ) 
 		}
 	} else{
 		tds <- lapply( 1:nreps, function(k) {
-			capture.output( { brk <- .boot.replicate(k) })
+			if (quiet)
+				capture.output( { brk <- .boot.replicate(k) })
+			else
+				brk <- .boot.replicate(k)
 			brk 
 		})
 	}
 	tds <- tds[!suppressWarnings ( sapply(tds, function(td) is.na(td[1]) ) )  ] 
 	if (length(tds)==0) stop('All bootstrap replicate failed with error.')
 	# output rate CIs, parameter CIs, trees
-	meanRates <- sapply( tds, function(td) td$meanRate )
+	meanRates <- sapply( tds, function(td) td$mean.rate )
 	cvs <- sapply( tds, function(td) td$coef_of_variation )
 	tmrcas <- sapply( tds, function(td) td$timeOfMRCA )
 	ttmrcas <- sapply( tds, function(td) td$timeTo )
@@ -345,7 +377,7 @@ boot <- function( td, tres,  ncpu = 1, searchRoot=1 , overrideTempConstraint=TRU
 	rv <- list( 
 		trees = tds
 		, meanRates = meanRates
-		, meanRate_CI = c( exp( log(td$meanRate) - log_mr_sd*1.96), exp(log(td$meanRate) + log_mr_sd*1.96 ))
+		, meanRate_CI = c( exp( log(td$meanRate) - log_mr_sd*1.96), exp(log(td$mean.rate) + log_mr_sd*1.96 ))
 		, coef_of_variation_CI = quantile( cvs, probs = c(alpha/2, 1 - alpha/2))
 		, timeOfMRCA_CI = timeOfMRCA_CI
 		, td = td
@@ -396,23 +428,24 @@ boot <- function( td, tres,  ncpu = 1, searchRoot=1 , overrideTempConstraint=TRU
 #' sts <- setNames( ape::node.depth.edgelength( tre )[1:ape::Ntip(tre)], tre$tip.label)
 #' # modify edge length to represent evolutionary distance with rate 1e-3:
 #' tre$edge.length <- tre$edge.length * 1e-3
-#' relaxedClockTest( tre, sts, nreps=25)
+#' relaxedClockTest( tre, sts, s= 1000,  omega0=.0015 , nreps=25)
 #'
 #'
 #' @export 
 relaxedClockTest <- function( ..., nreps=100, overrideTempConstraint=T , ncpu =1 )
 {
 	argnames <- names(list(...))
-	if ( 'strictClock'  %in% argnames) stop('Can not prespecify clock type *strictClock* for relaxed.clock.test. Quitting.')
-	td <- dater(..., strictClock=TRUE)
-	pbtd <-  parboot( td , nreps = nreps,  overrideTempConstraint=overrideTempConstraint
-	 , overrideClock = 'relaxed' , ncpu = ncpu )
+	if ( 'clock'  %in% argnames) stop('Can not prespecify clock type for relaxed.clock.test. Quitting.')
+	td <- dater(..., clock='strict')
+	suppressWarnings({ pbtd <-  parboot( td , nreps = nreps,  overrideTempConstraint=overrideTempConstraint
+	 , overrideClock = 'uncorrelated' , ncpu = ncpu )
+	})
 	cvci_null <- pbtd$coef_of_variation_CI
 	
-	tdrc <- dater(..., strictClock=FALSE)
+	tdrc <- dater(..., clock='uncorrelated')
 	clock <- 'strict'
 	if ( tdrc$coef_of_variation > cvci_null[2] ){
-		clock <- 'relaxed'
+		clock <- 'uncorrelated'
 	}
 	cat( paste( 'Best clock model: ', clock, '\n'))
 	cat( paste( 'Null distribution of rate coefficient of variation:', paste(cvci_null, collapse=' '), '\n'))
@@ -438,7 +471,7 @@ print.bootTreedater = print.boot.treedater <- function( x, ... )
 	 , paste( round(100*x$alpha/2, digits=3), '%')
 	 , paste( round(100*(1-x$alpha/2), digits=3), '%')
 	)
-	o <- data.frame( pseudoML= c(x$td$timeOfMRCA, x$td$meanRate)
+	o <- data.frame( pseudoML= c(x$td$timeOfMRCA, x$td$mean.rate)
 	 , c( x$timeOfMRCA_CI[1], x$meanRate_CI[1])  
 	 , c( x$timeOfMRCA_CI[2], x$meanRate_CI[2] )  
 	)
